@@ -1,12 +1,15 @@
 import datetime
 import logging
 import httpx
+import uuid
+import json
 
 from typing import Dict, Optional, overload, List
 
 from packaging.version import Version
 from typing_extensions import Unpack, Self
 
+from e2b.sandbox.mcp import McpServer
 from e2b.api.client.types import Unset
 from e2b.connection_config import ConnectionConfig, ApiParams
 from e2b.envd.api import ENVD_API_HEALTH_ROUTE, ahandle_envd_api_exception
@@ -114,6 +117,7 @@ class AsyncSandbox(SandboxApi):
             self.envd_api_url,
             self.connection_config,
             self._transport.pool,
+            self._envd_version,
         )
 
     async def is_running(self, request_timeout: Optional[float] = None) -> bool:
@@ -502,6 +506,7 @@ class AsyncSandbox(SandboxApi):
         envs: Optional[Dict[str, str]] = None,
         secure: bool = True,
         allow_internet_access: bool = True,
+        mcp: Optional[McpServer] = None,
         **opts: Unpack[ApiParams],
     ) -> Self:
         """
@@ -518,13 +523,19 @@ class AsyncSandbox(SandboxApi):
         :param envs: Custom environment variables for the sandbox
         :param secure: Envd is secured with access token and cannot be used without it, defaults to `True`.
         :param allow_internet_access: Allow sandbox to access the internet, defaults to `True`.
+        :param mcp: MCP server to enable in the sandbox
 
         :return: A Sandbox instance for the new sandbox
 
         Use this method instead of using the constructor to create a new sandbox.
         """
 
-        return await cls._create(
+        if not template and mcp is not None:
+            template = cls.default_mcp_template
+        elif not template:
+            template = cls.default_template
+
+        sandbox = await cls._create(
             template=template,
             timeout=timeout,
             auto_pause=auto_pause,
@@ -532,9 +543,24 @@ class AsyncSandbox(SandboxApi):
             envs=envs,
             secure=secure,
             allow_internet_access=allow_internet_access,
+            mcp=mcp,
             **opts,
         )
 
+        if mcp is not None:
+            token = str(uuid.uuid4())
+            sandbox._mcp_token = token
+
+            res = await sandbox.commands.run(
+                f"mcp-gateway --config '{json.dumps(mcp)}'",
+                user="root",
+                envs={"GATEWAY_ACCESS_TOKEN": token},
+            )
+            if res.exit_code != 0:
+                raise Exception(f"Failed to start MCP gateway: {res.stderr}")
+
+        return sandbox
+
     @overload
     async def beta_pause(
         self,
@@ -584,6 +610,20 @@ class AsyncSandbox(SandboxApi):
             **opts,
         )
 
+    async def beta_get_mcp_token(self) -> Optional[str]:
+        """
+        [BETA] This feature is in beta and may change in the future.
+
+        Get the MCP token for the sandbox.
+
+        :return: MCP token for the sandbox, or None if MCP is not enabled.
+        """
+        if not self._mcp_token:
+            self._mcp_token = await self.files.read(
+                "/etc/mcp-gateway/.token", user="root"
+            )
+        return self._mcp_token
+
     @classmethod
     async def _cls_connect(
         cls,
@@ -627,6 +667,7 @@ class AsyncSandbox(SandboxApi):
         metadata: Optional[Dict[str, str]],
         envs: Optional[Dict[str, str]],
         secure: bool,
+        mcp: Optional[McpServer] = None,
         **opts: Unpack[ApiParams],
     ) -> Self:
         extra_sandbox_headers = {}
@@ -646,6 +687,7 @@ class AsyncSandbox(SandboxApi):
                 env_vars=envs,
                 secure=secure,
                 allow_internet_access=allow_internet_access,
+                mcp=mcp,
                 **opts,
             )
 
